// TMesh.cpp: implementation of the TMesh class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "TMesh.h"


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

TMesh::TMesh()
{
	m_lpPoints = NULL;

	m_cx = 0;
	m_cy = 0;

	m_nWidth = 0;
	m_nHeight = 0;

	m_nDrawOffsetX = 0;      // ȭ鿡 ׸ Offset X
	m_nDrawOffsetY = 0;      // ȭ鿡 ׸ Offset Y

	m_nPointSize = 3;

	m_ptCtrl.x = 0;
	m_ptCtrl.y = 0;
	m_fIsSelected = FALSE;
}

TMesh::~TMesh()
{
	if(m_lpPoints)
		delete[] m_lpPoints;
}

// +----------------+
// |  HEADER        |
// |  POINT.x = cx  |
// |  POINT.y = cy  |
// +----------------+
// |  DATA          |
// |  POINTS        |
// |     .          |
// |     .          |
// |     .          |
// +----------------+
BOOL TMesh::Load( LPCTSTR lpszFileName, int nWidth, int nHeight, int nDrawOffsetX, int nDrawOffsetY )
{
	CFile	file;
	POINT	pt;
	int		cx, cy;
	LPPOINT lpPoint;
	int		x, y;
	int		dx, dy;

	if( !file.Open( lpszFileName, CFile::modeRead ) )
		return FALSE;

	if(m_lpPoints)
		delete[] m_lpPoints;

	file.Read( &pt, sizeof pt );
	cx = pt.x;
	cy = pt.y;
	dx = nWidth / (cx-1);
	dy = nHeight / (cy-1);

	m_lpPoints = (LPPOINT)new POINT[cx * cy];

	for(y = 0; y < cy; y++)
	{
		for(x = 0; x < cx; x++)
		{
			file.Read( &pt, sizeof pt);
			lpPoint = m_lpPoints + cx * y + x;
			lpPoint->x = pt.x;
			lpPoint->y = pt.y;
		}
	}

	//  κ  ̹ ũ⿡ .
	for(y = 0; y < cy; y++)
	{
		lpPoint = m_lpPoints + cx * y + (cx - 1);
		lpPoint->x = nWidth - 1;
	}

	for(x = 0; x < cx; x++)
	{
		lpPoint = m_lpPoints + cx * (cy - 1) + x;
		lpPoint->y = nHeight - 1;
	}

	m_cx = cx;
	m_cy = cy;
	m_nWidth = nWidth;
	m_nHeight = nHeight;
	m_nDrawOffsetX = nDrawOffsetX;
	m_nDrawOffsetY = nDrawOffsetY;

	file.Close();

	return TRUE;
}

BOOL TMesh::Save( LPCTSTR lpszFileName )
{
	CFile	file;
	POINT	pt;

	if( !file.Open( lpszFileName, CFile::modeCreate| CFile::modeWrite ) )
		return FALSE;

	pt.x = m_cx;
	pt.y = m_cy;

	file.Write( &pt, sizeof pt );
	file.Write( m_lpPoints, (sizeof pt) * m_cx * m_cy );
	file.Close();

	return TRUE;
}

BOOL TMesh::Create(int nWidth, int nHeight, int cx, int cy, int nDrawOffsetX, int nDrawOffsetY )
{
	if(m_lpPoints)
		delete[] m_lpPoints;

	m_lpPoints = (LPPOINT)new POINT[cx * cy];

	LPPOINT lpPoint;
	int x, y;
	int dx = nWidth / (cx-1);
	int dy = nHeight / (cy-1);

	for(y = 0; y < cy; y++)
	{
		for(x = 0; x < cx; x++)
		{
			lpPoint = m_lpPoints + cx * y + x;
			lpPoint->x = dx * x;
			lpPoint->y = dy * y;
		}
	}

	//  κ  ̹ ũ⿡ .
	for(y = 0; y < cy; y++)
	{
		lpPoint = m_lpPoints + cx * y + (cx - 1);
		lpPoint->x = nWidth - 1;
	}

	for(x = 0; x < cx; x++)
	{
		lpPoint = m_lpPoints + cx * (cy - 1) + x;
		lpPoint->y = nHeight - 1;
	}

	m_cx = cx;
	m_cy = cy;
	m_nWidth = nWidth;
	m_nHeight = nHeight;
	m_nDrawOffsetX = nDrawOffsetX;
	m_nDrawOffsetY = nDrawOffsetY;

	return TRUE;
}

void TMesh::Draw(HANDLE h, int xOffset, int yOffset)
{
	HDC	hdc = HDC( h );
	if(!m_lpPoints)
		return;

	LPPOINT lpPoint;
	int x, y;

	if(!hdc)
		return;

	HPEN hOldPen = (HPEN)SelectObject(hdc, GetStockObject(WHITE_PEN));

	SetROP2( hdc, R2_XORPEN );
	//  ε ׸.
	for(y = 0; y < m_cy; y++)
	{ 
		lpPoint = m_lpPoints + m_cx * y;
		MoveToEx(hdc, lpPoint->x, lpPoint->y, NULL);

		for(x = 1; x < m_cx; x++)
		{
			lpPoint = m_lpPoints + m_cx * y + x;
			LineTo(hdc, lpPoint->x, lpPoint->y);
		}
	}

	//   ׸.
	for(x = 0; x < m_cx; x++)
	{ 
		lpPoint = m_lpPoints + x;
		MoveToEx(hdc, lpPoint->x, lpPoint->y, NULL);

		for(y = 1; y < m_cy; y++)
		{
			lpPoint = m_lpPoints + m_cx * y + x;
			LineTo(hdc, lpPoint->x, lpPoint->y);
		}
	}

	SetROP2( hdc, R2_COPYPEN );
	//  ׸.
	RECT rc;
	LOGBRUSH LogFillBrush = {BS_SOLID, RGB(255,0,0), 0};
	LOGBRUSH LogFrameBrush = {BS_SOLID, RGB(0,0,0), 0};
	HBRUSH hFillBrush = CreateBrushIndirect(&LogFillBrush);
	HBRUSH hFrameBrush = CreateBrushIndirect(&LogFrameBrush);

	for(y = 0; y < m_cy; y++)
	{ 
		for(x = 0; x < m_cx; x++)
		{
			lpPoint = m_lpPoints + m_cx * y + x;

			rc.left   = lpPoint->x - m_nPointSize;
			rc.top    = lpPoint->y - m_nPointSize;
			rc.right  = lpPoint->x + m_nPointSize;
			rc.bottom = lpPoint->y + m_nPointSize;
			FillRect(hdc, &rc, hFillBrush);
			FrameRect(hdc, &rc, hFrameBrush);
		}
	}

	DeleteObject(hFillBrush);

	if( m_fIsSelected )
	{
		lpPoint = m_lpPoints + m_cx * m_ptCtrl.y + m_ptCtrl.x;
		hFillBrush = CreateSolidBrush( RGB( 0, 255, 0 ) );
		rc.left   = lpPoint->x - m_nPointSize;
		rc.top    = lpPoint->y - m_nPointSize;
		rc.right  = lpPoint->x + m_nPointSize;
		rc.bottom = lpPoint->y + m_nPointSize;
		FillRect(hdc, &rc, hFillBrush);
		FrameRect(hdc, &rc, hFrameBrush);
		DeleteObject( hFillBrush );
	}

	DeleteObject(hFrameBrush);

	if(hOldPen)
		SelectObject(hdc, hOldPen);
}

// lpPoint ġ  ִ Ȯ
// lpPoint ޽  x,y    ´.
BOOL TMesh::HitTest(int nScreenX, int nScreenY, LPPOINT lpCtrlPoint)
{
	POINT ptScreen = {nScreenX, nScreenY};
	LPPOINT lp;
	RECT rc;

	for(int y = 0; y < m_cy; y++)
	{ 
		for(int x = 0; x < m_cx; x++)
		{
			lp = m_lpPoints + m_cx * y + x;
			rc.left   = lp->x - m_nPointSize + m_nDrawOffsetX;
			rc.top    = lp->y - m_nPointSize + m_nDrawOffsetY;
			rc.right  = lp->x + m_nPointSize + m_nDrawOffsetX;
			rc.bottom = lp->y + m_nPointSize + m_nDrawOffsetY;

			if(PtInRect(&rc, ptScreen))
			{
				lpCtrlPoint->x = x;
				lpCtrlPoint->y = y;
				m_ptCtrl.x = x;
				m_ptCtrl.y = y;
				m_fIsSelected = TRUE;

				return TRUE;
			}
		}
	}

	m_fIsSelected = FALSE;
	return FALSE;
}

BOOL TMesh::SetMeshPoint(int nMeshX, int nMeshY, int nValX, int nValY)
{
	if(nMeshX < 0 || m_cx <= nMeshX)
		return FALSE;

	if(nMeshY < 0 || m_cy <= nMeshY)
		return FALSE;

	LPPOINT lpPoint = m_lpPoints + m_cx * nMeshY + nMeshX;
	lpPoint->x = nValX;
	lpPoint->y = nValY;

	return TRUE;
}

BOOL TMesh::GetMeshPoint(int nMeshX, int nMeshY, int& rValX, int& rValY)
{
	if(nMeshX < 0 || m_cx <= nMeshX)
		return FALSE;

	if(nMeshY < 0 || m_cy <= nMeshY)
		return FALSE;

	LPPOINT lpPoint = m_lpPoints + m_cx * nMeshY + nMeshX;
	rValX = lpPoint->x;
	rValY = lpPoint->y;

	return TRUE;
}


TMesh& TMesh::operator=(const TMesh& rMesh)
{
	if(&rMesh == this)
		return *this;

	if(m_lpPoints)
		delete [] m_lpPoints;

	int nPoints = rMesh.m_cx * rMesh.m_cy;
	m_lpPoints = (LPPOINT)new POINT[nPoints];

	for( int i = 0; i < nPoints; i++ )
		m_lpPoints[i] = rMesh.m_lpPoints[i];

	m_cx = rMesh.m_cx;
	m_cy = rMesh.m_cy;
	m_nWidth = rMesh.m_nWidth;
	m_nHeight = rMesh.m_nHeight;
	m_nDrawOffsetX = rMesh.m_nDrawOffsetX;
	m_nDrawOffsetY = rMesh.m_nDrawOffsetY;
	m_nPointSize = rMesh.m_nPointSize;

	return *this;
}

